@@ -1,6 +1,11 @@
+from __future__ import annotations
+
+from typing import List, Union
+
 import numpy as np
 
 from .points import Node
+from .transforms import Placement
 
 
 class CurveRevolve:
@@ -113,49 +118,23 @@ class CurvePoly:
         self._is_closed = is_closed
         self._debug = debug
 
-        from ada.core.utils import (
-            clockwise,
-            global_2_local_nodes,
-            local_2_global_nodes,
-            normal_to_points_in_plane,
-            unit_vector,
-        )
+        from ada.core.utils import clockwise, normal_to_points_in_plane, unit_vector
 
         if points2d is None and points3d is None:
             raise ValueError("Either points2d or points3d must be set")
 
+        if points2d is None:
+            normal = normal_to_points_in_plane([np.array(x[:3]) for x in points3d])
+            p1 = np.array(points3d[0][:3]).astype(float)
+            p2 = np.array(points3d[1][:3]).astype(float)
+            origin = p1
+            xdir = unit_vector(p2 - p1)
+
+        self._placement = Placement(origin, xdir=xdir, zdir=normal)
         if points2d is not None:
-            if origin is None or normal is None or xdir is None:
-                raise ValueError("You must supply origin, xdir and normal when passing in 2d points")
-            points2d_no_r = [n[:2] for n in points2d]
-            points3d = local_2_global_nodes(points2d_no_r, origin, xdir, normal)
-            for i, p in enumerate(points2d):
-                if len(p) == 3:
-                    points3d[i] = (
-                        points3d[i][0],
-                        points3d[i][1],
-                        points3d[i][2],
-                        p[-1],
-                    )
-                else:
-                    points3d[i] = tuple(points3d[i].tolist())
-            self._xdir = xdir
-            self._normal = np.array(normal)
-            self._origin = np.array(origin).astype(float)
-            self._ydir = np.cross(self._normal, self._xdir)
+            points3d = self._from_2d_points(points2d)
         else:
-            self._normal = normal_to_points_in_plane([np.array(x[:3]) for x in points3d])
-            self._origin = np.array(points3d[0][:3]).astype(float)
-            self._xdir = unit_vector(np.array(points3d[1][:3]) - np.array(points3d[0][:3]))
-            self._ydir = np.cross(self._normal, self._xdir)
-            csys = [self._xdir, self._ydir]
-            points2d = global_2_local_nodes(csys, self._origin, [np.array(x[:3]) for x in points3d])
-            points3d = [x.p if type(x) is Node else x for x in points3d]
-            for i, p in enumerate(points3d):
-                if len(p) == 4:
-                    points2d[i] = (points2d[i][0], points2d[i][1], p[-1])
-                else:
-                    points2d[i] = (points2d[i][0], points2d[i][1])
+            points2d = self._from_3d_points(points3d)
 
         if clockwise(points2d) is False:
             if is_closed:
@@ -169,7 +148,7 @@ class CurvePoly:
         self._points2d = points2d
 
         if flip_normal:
-            self._normal *= -1
+            self.placement.zdir *= -1
 
         self._seg_list = None
         self._seg_index = None
@@ -181,6 +160,38 @@ class CurvePoly:
         self._ifc_elem = None
         self._local2d_to_polycurve(points2d, tol)
 
+    def _from_2d_points(self, points2d) -> List[tuple]:
+        from ada.core.utils import local_2_global_nodes
+
+        place = self.placement
+
+        points2d_no_r = [n[:2] for n in points2d]
+        points3d = local_2_global_nodes(points2d_no_r, place.origin, place.xdir, place.zdir)
+        for i, p in enumerate(points2d):
+            if len(p) == 3:
+                points3d[i] = (
+                    points3d[i][0],
+                    points3d[i][1],
+                    points3d[i][2],
+                    p[-1],
+                )
+            else:
+                points3d[i] = tuple(points3d[i].tolist())
+        return points3d
+
+    def _from_3d_points(self, points3d) -> List[tuple]:
+        from ada.core.utils import global_2_local_nodes
+
+        csys = [self.placement.xdir, self.placement.ydir]
+        points2d = global_2_local_nodes(csys, self.placement.origin, [np.array(x[:3]) for x in points3d])
+        points3d = [x.p if type(x) is Node else x for x in points3d]
+        for i, p in enumerate(points3d):
+            if len(p) == 4:
+                points2d[i] = (points2d[i][0], points2d[i][1], p[-1])
+            else:
+                points2d[i] = (points2d[i][0], points2d[i][1])
+        return points2d
+
     def _generate_ifc_elem(self):
         a = self.parent.parent.get_assembly()
         f = a.ifc_file
@@ -202,30 +213,24 @@ class CurvePoly:
         return segindex
 
     def _local2d_to_polycurve(self, local_points2d, tol=1e-3):
-        """
-
-        :param local_points2d:
-        :param tol:
-        :return:
-        """
         from ada.core.curve_utils import build_polycurve, segments_to_indexed_lists
         from ada.core.utils import local_2_global_nodes
 
         debug_name = self._parent.name if self._parent is not None else "PolyCurveDebugging"
 
         seg_list = build_polycurve(local_points2d, tol, self._debug, debug_name)
-
+        origin, xdir, normal = self.placement.origin, self.placement.xdir, self.placement.zdir
         # # Convert from local to global coordinates
         for i, seg in enumerate(seg_list):
             if type(seg) is ArcSegment:
                 lpoints = [seg.p1, seg.p2, seg.midpoint]
-                gp = local_2_global_nodes(lpoints, self.origin, self.xdir, self.normal)
+                gp = local_2_global_nodes(lpoints, origin, xdir, normal)
                 seg.p1 = gp[0]
                 seg.p2 = gp[1]
                 seg.midpoint = gp[2]
             else:
                 lpoints = [seg.p1, seg.p2]
-                gp = local_2_global_nodes(lpoints, self.origin, self.xdir, self.normal)
+                gp = local_2_global_nodes(lpoints, origin, xdir, normal)
                 seg.p1 = gp[0]
                 seg.p2 = gp[1]
 
@@ -233,10 +238,23 @@ class CurvePoly:
         self._seg_global_points, self._seg_index = segments_to_indexed_lists(seg_list)
         self._nodes = [Node(p) if len(p) == 3 else Node(p[:3], r=p[3]) for p in self._points3d]
 
+    def _update_curves(self):
+        from ada.core.utils import local_2_global_nodes
+
+        points2d_no_r = [n[:2] for n in self.points2d]
+        points3d = local_2_global_nodes(points2d_no_r, self.placement.origin, self.placement.xdir, self.placement.zdir)
+        for i, p in enumerate(self.points2d):
+            if len(p) == 3:
+                points3d[i] = (points3d[i][0], points3d[i][1], points3d[i][2], p[-1])
+            else:
+                points3d[i] = tuple(points3d[i].tolist())
+        self._points3d = points3d
+        self._local2d_to_polycurve(self.points2d, tol=self._tol)
+
     def make_extruded_solid(self, height: float):
         from ada.occ.utils import extrude_closed_wire
 
-        return extrude_closed_wire(self.wire, self.origin, self.normal, height)
+        return extrude_closed_wire(self.wire, self.placement.origin, self.placement.zdir, height)
 
     def make_revolve_solid(self, axis, angle, origin):
         from ada.occ.utils import make_revolve_solid
@@ -248,7 +266,7 @@ class CurvePoly:
 
         return wire_to_face(self.edges)
 
-    def calc_bbox(self, thick):
+    def calc_bbox(self, tol):
         """
         Calculate the Bounding Box of the plate
 
@@ -272,11 +290,11 @@ class CurvePoly:
         matr = {0: "X", 1: "Y", 2: "Z"}
         orient = matr[pv[0]]
         if orient == "X" or orient == "Y":
-            delta_vec = abs(n * thick / 2.0)
+            delta_vec = abs(n * tol / 2.0)
             bbox_min -= delta_vec
             bbox_max += delta_vec
         elif orient == "Z":
-            delta_vec = abs(n * thick).astype(np.float64)
+            delta_vec = abs(n * tol).astype(np.float64)
             bbox_min -= delta_vec
 
         else:
@@ -285,29 +303,19 @@ class CurvePoly:
         return tuple([(x, y) for x, y in zip(list(bbox_min), list(bbox_max))])
 
     def scale(self, scale_factor, tol):
-        self._origin = np.array([x * scale_factor for x in self.origin])
+        self.placement.origin = np.array([x * scale_factor for x in self.placement.origin])
         self._points2d = [tuple([x * scale_factor for x in p]) for p in self._points2d]
         self._points3d = [tuple([x * scale_factor for x in p]) for p in self._points3d]
         self._local2d_to_polycurve(self.points2d, tol=tol)
 
     @property
-    def origin(self):
-        return self._origin
+    def placement(self) -> Placement:
+        return self._placement
 
-    @origin.setter
-    def origin(self, value):
-        from ada.core.utils import local_2_global_nodes
-
-        self._origin = value
-        points2d_no_r = [n[:2] for n in self.points2d]
-        points3d = local_2_global_nodes(points2d_no_r, self._origin, self.xdir, self.normal)
-        for i, p in enumerate(self.points2d):
-            if len(p) == 3:
-                points3d[i] = (points3d[i][0], points3d[i][1], points3d[i][2], p[-1])
-            else:
-                points3d[i] = tuple(points3d[i].tolist())
-        self._points3d = points3d
-        self._local2d_to_polycurve(self.points2d, tol=self._tol)
+    @placement.setter
+    def placement(self, value: Placement):
+        self._placement = value
+        self._update_curves()
 
     @property
     def seg_global_points(self):
@@ -318,24 +326,24 @@ class CurvePoly:
         return self._points2d
 
     @property
-    def points3d(self):
+    def points3d(self) -> List[float]:
         return self._points3d
 
     @property
-    def nodes(self):
+    def nodes(self) -> List[Node]:
         return self._nodes
 
     @property
     def normal(self):
-        return self._normal
+        return self.placement.zdir
 
     @property
     def xdir(self):
-        return self._xdir
+        return self.placement.xdir
 
     @property
     def ydir(self):
-        return self._ydir
+        return self.placement.ydir
 
     @property
     def edges(self):
@@ -358,7 +366,7 @@ class CurvePoly:
         return self._seg_index
 
     @property
-    def seg_list(self):
+    def seg_list(self) -> List[Union[LineSegment, ArcSegment]]:
         return self._seg_list
 
     @property
@@ -383,7 +391,7 @@ class LineSegment:
         self._edge_geom = edge_geom
 
     @property
-    def p1(self):
+    def p1(self) -> np.ndarray:
         if type(self._p1) is not np.ndarray:
             self._p1 = np.array(self._p1)
         return self._p1
@@ -393,7 +401,7 @@ class LineSegment:
         self._p1 = value
 
     @property
-    def p2(self):
+    def p2(self) -> np.ndarray:
         if type(self._p2) is not np.ndarray:
             self._p2 = np.array(self._p2)
         return self._p2
@@ -428,7 +436,7 @@ class ArcSegment(LineSegment):
         self._midpoint = value
 
     @property
-    def radius(self):
+    def radius(self) -> float:
         return self._radius
 
     @radius.setter
