@@ -1,8 +1,12 @@
 sesam_el_map = {
     15: "B31",
     2: "B31",
+    23: "B32",
     24: "S4R",
     25: "S3",
+    26: "STRI65",
+    28: "S8R",
+    31: "C3D10",
     40: "SPRING2",
     18: "SPRING1",
     11: "MASS",
