@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import List, Union
 
 from .common import FemBase
@@ -13,6 +15,48 @@ class HistTypes:
     all = [NODE, ENERGY, CONTACT, CONNECTOR]
 
 
+class IntervalTypes:
+    FREQUENCY = "frequency"
+
+
+class HistDataTypes:
+    ALLAE = "ALLAE"
+    ALLCD = "ALLCD"
+    ALLDMD = "ALLDMD"
+    ALLEE = "ALLEE"
+    ALLFD = "ALLFD"
+    ALLIE = "ALLIE"
+    ALLJD = "ALLJD"
+    ALLKE = "ALLKE"
+    ALLKL = "ALLKL"
+    ALLPD = "ALLPD"
+    ALLQB = "ALLQB"
+    ALLSD = "ALLSD"
+    ALLSE = "ALLSE"
+    ALLVD = "ALLVD"
+    ALLWK = "ALLWK"
+    ETOTAL = "ETOTAL"
+
+    all = [
+        ALLAE,
+        ALLCD,
+        ALLDMD,
+        ALLEE,
+        ALLFD,
+        ALLIE,
+        ALLJD,
+        ALLKE,
+        ALLKL,
+        ALLPD,
+        ALLQB,
+        ALLSD,
+        ALLSE,
+        ALLVD,
+        ALLWK,
+        ETOTAL,
+    ]
+
+
 class HistOutput(FemBase):
     """
 
@@ -24,25 +68,8 @@ class HistOutput(FemBase):
     """
 
     TYPES = HistTypes
-
-    default_hist = [
-        "ALLAE",
-        "ALLCD",
-        "ALLDMD",
-        "ALLEE",
-        "ALLFD",
-        "ALLIE",
-        "ALLJD",
-        "ALLKE",
-        "ALLKL",
-        "ALLPD",
-        "ALLQB",
-        "ALLSD",
-        "ALLSE",
-        "ALLVD",
-        "ALLWK",
-        "ETOTAL",
-    ]
+    TYPES_INTERVAL = IntervalTypes
+    TYPES_DATA = HistDataTypes
 
     def __init__(
         self,
@@ -51,7 +78,7 @@ class HistOutput(FemBase):
         set_type: str,
         variables: List[str],
         int_value=1,
-        int_type="frequency",
+        int_type=TYPES_INTERVAL.FREQUENCY,
         metadata=None,
         parent=None,
     ):
@@ -177,3 +204,8 @@ class FieldOutput(FemBase):
         if value.upper() not in FieldOutput._valid_fstep_type:
             raise ValueError(f'Field output step type "{value}" is not supported')
         self._int_type = value.upper()
+
+
+class Defaults:
+    history_output = HistOutput("default_hist", None, HistOutput.TYPES.ENERGY, HistOutput.TYPES_DATA.all)
+    field_output = FieldOutput("default_fields", int_type=HistOutput.TYPES_INTERVAL.FREQUENCY, int_value=1)
